/**
    @file

    @brief
	(Brief Text)

	(Description Text)
**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2015, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

/* INCLUDES ************************************************************************/
#include "ws2812.h"

/* CONSTANTS ***********************************************************************/
#define WS2812_GPIO_SYM0 (0xE0)
#define WS2812_GPIO_SYM1 (0xF8)
#define WS2812_GPIO (41)

/* GLOBAL VARIABLES ****************************************************************/

/* LOCAL VARIABLES *****************************************************************/

/* MACROS **************************************************************************/

/* LOCAL FUNCTIONS / INLINES *******************************************************/

void ws2812_shiftout(uint8_t b);

/* FUNCTIONS ***********************************************************************/

void ws2812_shiftout(uint8_t b)
{
    uint8_t i;
    register uint8_t sym = 0, sym0 = WS2812_GPIO_SYM0, sym1 = WS2812_GPIO_SYM1;
    register uint32_t mask = 0;
    register volatile uint32_t* gpioval;

#if (WS2812_GPIO < 32)
    /* 0 - 31 */
    gpioval = &(GPIO->GPIO00_31_VAL);
    mask = 1 << WS2812_GPIO;
#elif (WS2812_GPIO < 64)
    /* 32 - 63 */
    gpioval = &(GPIO->GPIO32_63_VAL);
    mask = 1 << (WS2812_GPIO - 32);
#else
    /* 64 - 66 */
    gpioval = &(GPIO->GPIO64_66_VAL);
    mask = 1 << (WS2812_GPIO - 64);
#endif

    for(i = 0; i < 8; ++i)
    {
        if(b & 0x80)    sym = sym1;
        else            sym = sym0;

        if (sym & 0x80) *gpioval |= mask;
        else            *gpioval &= ~mask;
        asm_noop();
        asm_noop();
        asm_noop();
        asm_noop();
        if (sym & 0x40) *gpioval |= mask;
        else            *gpioval &= ~mask;
        asm_noop();
        asm_noop();
        asm_noop();
        if (sym & 0x20) *gpioval |= mask;
        else            *gpioval &= ~mask;
        asm_noop();
        asm_noop();
        asm_noop();
        asm_noop();
        if (sym & 0x10) *gpioval |= mask;
        else            *gpioval &= ~mask;
        asm_noop();
        asm_noop();
        asm_noop();
        if (sym & 0x08) *gpioval |= mask;
        else            *gpioval &= ~mask;
        asm_noop();
        asm_noop();
        asm_noop();
        asm_noop();
        if (sym & 0x04) *gpioval |= mask;
        else            *gpioval &= ~mask;
        asm_noop();
        asm_noop();
        asm_noop();
        if (sym & 0x02) *gpioval |= mask;
        else            *gpioval &= ~mask;
        asm_noop();
        asm_noop();
        asm_noop();
        asm_noop();
        if (sym & 0x01) *gpioval |= mask;
        else            *gpioval &= ~mask;

        b <<= 1;
    }
}

void ws2812_init(void)
{
    gpio_function(WS2812_GPIO, pad_func_0);
    gpio_dir(WS2812_GPIO, pad_dir_output);
}

void ws2812_write(uint8_t r, uint8_t g, uint8_t b)
{
    ws2812_shiftout(g);
    ws2812_shiftout(r);
    ws2812_shiftout(b);
}
